import { Component, OnInit, OnDestroy } from '@angular/core';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { AppNativeWrapperService } from '../../services/app-native-wrapper/app-native-wrapper.service';
import { LocalFile } from './localFile';
import { ElectronService } from 'ngx-electron';
import { runInThisContext } from 'vm';

@Component({
	selector: 'app-home',
	templateUrl: './home.component.html',
	styleUrls: ['./home.component.scss']
})
export class HomeComponent implements OnInit, OnDestroy {
	public isNativeWrapper: boolean = false;
	public electronVersion: string = 'UNKNOWN';
	public zoomPercentageOptions: JhaButtonGroupOption[] = [];
	public zoomPercentage: string = '100';
	public progressValue: number = 0;
	public fileList: LocalFile[] = [];
	public filePath: string = '';
	public pathSeparator: string = '\\';
	private timerHandle: number = -1;

	constructor(
		private electronService: ElectronService,
		private appNativeWrapperService: AppNativeWrapperService) {
	}

	ngOnInit() {

		if (this.electronService.isElectronApp) {
			// We're in Electron
			this.isNativeWrapper = true;

			this.updateZoomPercentage();

			// Get path separator
			this.appNativeWrapperService.getPathSeparator().then((pathSeparator) => {
				this.pathSeparator = pathSeparator;
			});

			// Get home directory
			this.appNativeWrapperService.getHomeDir().then((homeDir) => {
				this.filePath = homeDir;
			});

			// Get Electron version
			this.appNativeWrapperService.getElectronVersion().then((version) => {
				this.electronVersion = version;
			});

			// Get local files for C Drive
			this.getLocalFiles();

			setTimeout(() => {
				console.log('isNativeWrapper = ' + this.isNativeWrapper);
				console.log('filePath = ' + this.filePath);
				console.log('electronVersion = ' + this.electronVersion);
			}, 500);
		}

		this.zoomPercentageOptions = [
			new JhaButtonGroupOption('75', '75', '', !this.isNativeWrapper),
			new JhaButtonGroupOption('100', '100', '', !this.isNativeWrapper),
			new JhaButtonGroupOption('125', '125', '', !this.isNativeWrapper),
			new JhaButtonGroupOption('150', '150', '', !this.isNativeWrapper)
		];

	}

	ngOnDestroy() {
		// Reset taskbar progress when leaving this page
		this.clearProgress();
	}

	public launchCalculator() {
		// Call service to launch calculator
		this.appNativeWrapperService.launchCalculator();
	}

	public updateZoomPercentage() {
		this.appNativeWrapperService.setZoomLevel(parseInt(this.zoomPercentage, 10));
	}

	public startProgress() {
		// Start progress out at 9
		this.progressValue = 0;

		// Set up timer to mock up progress
		this.timerHandle = window.setInterval(() => {

			// Dummy up new progress bar value by incrementing by 10
			this.progressValue += 10;

			// Call service to set the native taskbar progress value
			this.appNativeWrapperService.setTaskbarProgress(this.progressValue / 100);

			// Turn off interval when we hit 100%
			if (this.progressValue >= 100) {
				window.clearInterval(this.timerHandle);
				this.timerHandle = -1;
			}

		}, 1000);
	}

	public clearProgress() {
		// Stop timer if it's running
		if (this.timerHandle >= 0) {
			window.clearInterval(this.timerHandle);
		}

		// Reset progress value
		this.progressValue = 0;

		// Call service to reset taskbar progress
		this.appNativeWrapperService.setTaskbarProgress(-1);
	}

	public selectFolder(pathName: string) {
		this.updateFolder(this.filePath += this.pathSeparator + pathName);
	}

	public upOneFolder() {
		const i = this.filePath.lastIndexOf(this.pathSeparator);
		if (i >= 0) {
			this.updateFolder(this.filePath.substr(0, i));
		}
	}

	private updateFolder(newFolderPath: string) {
		this.filePath = newFolderPath;
		this.getLocalFiles();
	}

	private getLocalFiles() {
		this.appNativeWrapperService.localFiles(this.filePath).then((files) => {
			// Parse list of local files from service call result
			this.fileList = [];
			for (let i = 0; i < files.length; i++) {
				this.fileList.push(
					new LocalFile(
						(files[i] as any).name,
						(files[i] as any).type
					)
				);
			}

			// Sort the list first on file type then on file name
			this.fileList.sort((itemA: LocalFile, itemB: LocalFile) => {
				let sortResult = 0;
				if (itemA.type < itemB.type) {
					sortResult = -1;
				} else if (itemA.type > itemB.type) {
					sortResult = 1;
				} else if (itemA.name.toLowerCase() < itemB.name.toLowerCase()) {
					sortResult = -1;
				} else if (itemA.name.toLowerCase() > itemB.name.toLowerCase()) {
					sortResult = 1;
				}

				return sortResult;
			});

		});
	}

	public openPattern(url: string) {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
