// See the Native Wrapper pattern for more details on using Electron to
// host JHA apps: http://jhaux.com/responsive/#/Patterns/NativeWrapper

// Imports
const { app, BrowserWindow, ipcMain } = require('electron');
const path = require('path');
const { exec } = require('child_process');
const os = require('os');

// Keep a global reference to the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let win

// Create the browser window and load your web app
function createWindow() {

	// Create the browser window
	let newWindow = new BrowserWindow({

		// Don't show the window initially. We do that in the 'ready-to-show' 
		// event handler below.
		show: false,

		// Window size: should be large enough to comfortably display your app
		// without being larger than a typical user's monitor.
		width: 1400,
		height: 900,

		// Initial window background color; leave at this value for consistency
		// with other JHA apps.
		backgroundColor: '#333333',

		// Make the window resizable by the user.
		resizable: true,

		// The path to the icon file displayed for your app by the OS. This file
		// must be in ICO format and should be at least 256x256 for best results.
		icon: path.join(__dirname, 'icon.ico'),

		// IMPORTANT! Node integration MUST be turned off and a preload script used to
		// facilitate IPC *unless* your app will ALWAYS be fully embedded within Electron
		// and not hosted on a server, i.e you always specify the file:// protocol
		// for your web app and not  the http(s):// protocol. Enabling node integration
		// in a hosted web app is a serious XSS security risk.
		// https://electronjs.org/docs/tutorial/security#2-disable-nodejs-integration-for-remote-content
		webPreferences: {
			nodeIntegration: false,
			preload: path.join(__dirname, 'preload.js')
		}
	})

	// --------------------------------------------------------------
	// Load your web app into the Electron window.
	//
	// Your app can either be local (fully embedded with Electron)
	// and use the file:// protocol to access it, or it can be hosted
	// on a web server, in which case you use the http(s):// protocol 
	// to access it.
	// 
	// It can be useful to embed a local version of an Angular app 
	// while debugging it within Electron, but in production most
	// JHA products will host their app on a web server to make
	// updates easier. The only way to update a local app is with
	// full or partial installer updates. Updates can be made to a 
	// hosted app on the server as needed, without requiring updates
	// to your Electron deployment.
	//
	// Load your app using one of the following two options.
	// --------------------------------------------------------------

	// OPTION 1 (preferred for production): Most JHA products will
	// host their app on a web server since the update process is
	// much easier. Here's an example of loading a web app hosted
	// on a server. Comment this out if you use option 2.
	//newWindow.loadURL('https://my-server-based-application-url');

	// OPTION 2: Use the file:// protocol if your app will be fully
	// embedded within the Electron window and not hosted. This can
	// be useful when debugging an Angular app within Electron.
	// Here's an example of loading a web app locally. Comment this
	// out if you use option 1.
	newWindow.loadURL(`file://${__dirname}/index.html`);

	// --------------------------------------------------------------
	// Window management messages
	// --------------------------------------------------------------

	// Emitted when the app is ready to show. Do not change or remove!
	newWindow.once('ready-to-show', () => {

		// Maximize the window: this is optional and can be removed
		newWindow.maximize();

		// Show the window
		newWindow.show();
	})

	// Emitted when the window is closed. Do not change or remove!
	newWindow.on('closed', () => {
		// Dereference the window object, usually you would store windows
		// in an array if your app supports multi windows, this is the time
		// when you should delete the corresponding element.
		newWindow = null
	})

	// Let the renderer process know when the app is trying to open an 
	// external link.  When it does, prevent the default behavior and
	// open the link in the users default browser. Do not change or remove!
	newWindow.webContents.on('new-window', function (e, url) {
		e.preventDefault();
		require('electron').shell.openExternal(url);
	})

	return newWindow;
}

// --------------------------------------------------------------
// Window management messages
// --------------------------------------------------------------

// This method will be called when Electron has finished initialization and
// is ready to create browser windows. Some APIs can only be used AFTER this
// event occurs. Do not change or remove!
app.on('ready', () => {
	win = createWindow();
})

// Quit when all windows are closed. Do not change or remove!
app.on('window-all-closed', () => {
	// On macOS it is common for applications and their menu bar
	// to stay active until the user quits explicitly with Cmd+Q.
	if (process.platform !== 'darwin') {
		app.quit();
	}
})

// On macOS it's common to re-create a window in the app when the
// dock icon is clicked and there are no other windows open.
app.on('activate', () => {
	if (win === null) {
		win = createWindow();
	}
})

// --------------------------------------------------------------
// Put your app's custom Electron functionality below here
// --------------------------------------------------------------

// Request path separator
ipcMain.on('req-path-separator', (event, arg) => {
	event.sender.send('res-path-separator', path.sep);
});

// Request for home directory
ipcMain.on('req-home-dir', (event, arg) => {
	event.sender.send('res-home-dir', os.homedir());
});

// Request for electron version
ipcMain.on('req-electron-version', (event, arg) => {
	// process.versions.electron contains the Electron version
	event.sender.send('res-electron-version', process.versions.electron);
});

// Request to set the taskbar progress indicator to the specified value
ipcMain.on('req-set-taskbar-progress', (event, arg) => {
	// Set the progress indicator in the taskbar icon
	win.setProgressBar(arg)
});

// Request for local files
ipcMain.on('req-local-files', (event, arg) => {
	const fs = require('fs');
	const path = require("path");
	var fileArray = [];
	var dir = arg + path.sep;

	// Gather a list of files and folders in the specified folder
	fs.readdir(dir, function (err, items) {

		if (items && items.length > 0) {
			for (var i = 0; i < items.length; i++) {
				var file = items[i];
				var type = "file";

				try {
					if (fs.statSync(path.join(dir, file)).isDirectory())
						type = "dir";
				}
				catch (err) {
				}

				fileArray.push({ "name": file, "type": type });
			}
			fs.stat(file, file_callback(fileArray));
		} else {
			fileArray.push({ "name": "This folder is empty.", "type": "empty" });
			fs.stat("This folder is empty.", file_callback(fileArray));
		}
	});

	// Send the result in a response message
	function file_callback(fileArray) {
		return function (err, stats) {
			event.sender.send('res-local-files', fileArray);
		}
	};
});
